import java.io.*;
import java.util.*;

class Compilatore
{
	public static void main(String[] args)
	{
		String riga_sorgente;
		int num_riga = 1;
		
		if (args.length == 0)
		{
			System.out.println("Uso: java Compilatore programma.lurm");
			return;
		}
		
		try
		{
			BufferedReader input = new BufferedReader(new FileReader(args[0]));
			
			scrivi_intestazione();
			
			while ((riga_sorgente = input.readLine()) != null)
				compila_riga(num_riga++, riga_sorgente);
			
			scrivi_conclusione();
			
			input.close();
		}
		catch (Exception e)
		{
			System.err.println("Impossibile leggere il file: " + e.getMessage());
			System.exit(1);
		}
	}
	
	public static void scrivi_intestazione()
	{
		System.out.println("public int eseguiComputazione(int[] input)");
		System.out.println("{");
		System.out.println("\tint PC = 1; /* Contatore di programma */");
		System.out.println("\tint R[] = new int[100]; /* Registri */");
		System.out.println("\tboolean in_esecuzione = true;");
		System.out.println("\t");
		System.out.println("\t/* Tutti i registri non ricevuti in input vengono azzerati */");
		System.out.println("\tfor (int i = 0; i < R.length; i++)");
		System.out.println("\t\tR[i] = (i < input.length) ? input[i] : 0;");
		System.out.println("\t");
		System.out.println("\t/* Questo ciclo serve a simulare l'istruzione goto, non presente in Java */");
		System.out.println("\twhile (in_esecuzione)");
		System.out.println("\t{");
		System.out.println("\t\t/* PC corrisponde alla riga del codice sorgente */");
		System.out.println("\t\tswitch (PC)");
		System.out.println("\t\t{");
	}
	
	public static void compila_riga(int num_riga, String urm_istruz)
	{
		int reg1, reg2, virg1_pos, virg2_pos;
		
		System.out.println("\t\t\tcase " + num_riga + ": /* " + urm_istruz + " */");
		
		switch (urm_istruz.charAt(0))
		{
			case 'Z':
				reg1 = Integer.parseInt(urm_istruz.substring(2, urm_istruz.length() - 1));
				System.out.println("\t\t\t\tR[" + (reg1-1) + "] = 0;");
				break;
			case 'S':
				reg1 = Integer.parseInt(urm_istruz.substring(2, urm_istruz.length() - 1));
				System.out.println("\t\t\t\tR[" + (reg1-1) + "]++;");
				break;
			case 'T':
				virg1_pos = urm_istruz.indexOf(',');
				reg1 = Integer.parseInt(urm_istruz.substring(2, virg1_pos));
				reg2 = Integer.parseInt(urm_istruz.substring(virg1_pos + 1, urm_istruz.length() - 1));
				System.out.println("\t\t\t\tR[" + (reg2-1) + "] = R[" + (reg1-1) + "];");
				break;
			case 'J':
				virg1_pos = urm_istruz.indexOf(',');
				virg2_pos = urm_istruz.lastIndexOf(',');
				reg1 = Integer.parseInt(urm_istruz.substring(2, virg1_pos));
				reg2 = Integer.parseInt(urm_istruz.substring(virg1_pos + 1, virg2_pos));
				
				/* Semplifica il caso in cui un registro viene confrontato con se stesso */
				if (reg1 == reg2)
				{
					System.out.println("\t\t\t\tPC = " + urm_istruz.substring(virg2_pos + 1, urm_istruz.length() - 1) + "; /* salto incondizionato */");
					System.out.println("\t\t\t\tbreak;");
				}
				else
				{
					System.out.println("\t\t\t\tif (R[" + (reg1-1) + "] == R[" + (reg2-1) + "]) /* salto condizionato */");
					System.out.println("\t\t\t\t{");
					System.out.println("\t\t\t\t\tPC = " + urm_istruz.substring(virg2_pos + 1, urm_istruz.length() - 1) + ";");
					System.out.println("\t\t\t\t\tbreak;");
					System.out.println("\t\t\t\t}");
				}
				break;
		}
	}
	
	public static void scrivi_conclusione()
	{
		System.out.println("\t\t\tdefault:");
		System.out.println("\t\t\t\tin_esecuzione = false; /* Termina la computazione */");
		System.out.println("\t\t\t\tbreak;");
		System.out.println("\t\t}");
		System.out.println("\t}");
		System.out.println("\t");
		System.out.println("\treturn R[0]; /* Restituisce il valore del primo registro */");
		System.out.println("}");
	}
}
