import java.io.*;
import java.util.*;

class Interprete
{
	public static void main(String[] args)
	{
		int virgola1_pos, virgola2_pos;
		boolean in_esecuzione = true;
		
		/* Memoria programma (vettore di stringhe) */
		Vector<String> programma = new Vector<String>();
		
		/* Controllo di sequenza */
		int PC = 1;		// Program counter
		String istruz;		// Istruzione da eseguire
		
		/* Variabili usate per la decodifica */
		char tipo_istruz=0;	// Tipo di istruzione (Z, S, T, J oppure H)
		int op1=0, op2=0, op3=0;// Operandi dell'istruzione da eseguire
		int registri_da_leggere=0;
		
		/* Variabili usate per leggere e scrivere i valori dei registri */
		int load1=0, load2=0, store_valore=0, store_registro=0;
		boolean istruz_produce_output=false;
		
		/* Registri URM (memoria dati) */
		int[] R = new int[100];
		
		if (args.length == 0)
		{
			System.out.println("Uso: java Interprete programma.lurm [parametri...]");
			return;
		}
		
		/* Carica file da interpretare nella memoria programma */
		try
		{
			BufferedReader input = new BufferedReader(new FileReader(args[0]));
			String riga;
			
			while ((riga = input.readLine()) != null)
				programma.add(riga);
			
			input.close();
		}
		catch (Exception e)
		{
			System.err.println("Impossibile leggere il file: " + e.getMessage());
			System.exit(1);
		}
		
		/* Inizializzazione dei registri. Tutti i registri non ricevuti in input vengono azzerati */
		for (int i = 0; i < R.length; i++)
			R[i] = 0;
		for (int i = 1; i < args.length; i++)
			R[i-1] = Integer.parseInt(args[i]);
		
		while (in_esecuzione)
		{
			// Fetch (legge l'istruzione alla PC-esima riga del programma)
			// PC è diminuito di uno perché in Java i vettori sono indicizzati a partire da zero
			istruz = (PC <= programma.size()) ? programma.get(PC-1) : null;
			
			// Decodifica
			if (istruz == null)
			{
				tipo_istruz = 'H';	// Halt
				registri_da_leggere = 0;
				istruz_produce_output = false;
			}
			else switch (istruz.charAt(0))
			{
				case 'Z':
					tipo_istruz = 'Z';
					op1 = Integer.parseInt(istruz.substring(2, istruz.length() - 1));
					registri_da_leggere = 1;
					istruz_produce_output = true;
					break;
				case 'S':
					tipo_istruz = 'S';
					op1 = Integer.parseInt(istruz.substring(2, istruz.length() - 1));
					registri_da_leggere = 1;
					istruz_produce_output = true;
					break;
				case 'T':
					tipo_istruz = 'T';
					virgola1_pos = istruz.indexOf(',');
					op1 = Integer.parseInt(istruz.substring(2, virgola1_pos));
					op2 = Integer.parseInt(istruz.substring(virgola1_pos + 1, istruz.length() - 1));
					registri_da_leggere = 2;
					istruz_produce_output = true;
					break;
				case 'J':
					tipo_istruz = 'J';
					virgola1_pos = istruz.indexOf(',');
					virgola2_pos = istruz.lastIndexOf(',');
					op1 = Integer.parseInt(istruz.substring(2, virgola1_pos));
					op2 = Integer.parseInt(istruz.substring(virgola1_pos + 1, virgola2_pos));
					op3 = Integer.parseInt(istruz.substring(virgola2_pos + 1, istruz.length() - 1));
					registri_da_leggere = 2;
					istruz_produce_output = false;
					break;
			}
			
			// Acquisizione operandi.
			// L'indice del registro viene diminuito di uno perché in Java gli elementi
			// dell'array sono numerati a partire da zero
			if (registri_da_leggere >= 1)
				load1 = R[op1-1];
			if (registri_da_leggere >= 2)
				load2 = R[op2-1];
			
			// Esecuzione
			switch (tipo_istruz)
			{
				case 'H':
					in_esecuzione = false;	/* Termina l'esecuzione */
					break;
				case 'Z':
					store_registro = op1;
					store_valore = 0;
					PC++;
					break;
				case 'S':
					store_registro = op1;
					store_valore = load1 + 1;
					PC++;
					break;
				case 'T':
					store_registro = op2;
					store_valore = load1;
					PC++;
					break;
				case 'J':
					if (load1 == load2)
						PC = op3;
					else
						PC++;
					break;
			}
			
			// Memorizzazione risultato
			if (istruz_produce_output)
				R[store_registro-1] = store_valore;
		}
		
		System.out.println("Output della computazione: " + R[0]);
	}
}
